/***************************************************************************
 *                                                                          *
 * lcd.c                                                                    *
 * This is lcd driver program file for SVT17702.             				*
 * History:  2008/04/01  Start.                                             *
 *                                                                          *
 ***************************************************************************/

/* #include */
#include "s1c17702.h"
#include "lcd.h"

/* Extern */
extern void pwb_wait_iosc(void);
extern void pwb_wait_osc3(void);
extern void pwb_wait_osc1(void);

/* Functions */
/*************************************************************   
 * initLcdPower function.
 * This function initializes LCD power.
 * param	: unsigned char boosterSelect	power voltage booster. 0:off  1:on
 * param	: unsigned char protectSelect	LCD heavy load protection mode. 0:off  1:on
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void initLcdPower(unsigned char boosterSelect,unsigned char protectSelect) {
	
	if(boosterSelect == LCD_PWR_BOOSTER_ON) {
		// if LCD power booster turns on...
		// Power voltage booster control = on (D[0]/0x50a4)
		LCD_PWR_PBON = 1;

		// wait till LCD power booster is stable
		if(OSC_SRC_CLKSRC == 0) {
			if(OSC_SRC_HSCLKSEL == 0) {
				// if system clock is IOSC
				pwb_wait_iosc();			
			}
			else {
				// if system clock is OSC3
				pwb_wait_osc3();
			}
		} else {
			// if system clock is OSC1
			pwb_wait_osc1();
		}
		// Regulator power source select = VD2 (D[1]/0x50a4)
		LCD_PWR_VDSEL = 1;
	} else {
		// if LCD power booster turns off...
		// Power voltage booster control = off (D[0]/0x50a4)
		LCD_PWR_PBON = 0;

		// Regulator power source select = VDD (D[1]/0x50a4)
		LCD_PWR_VDSEL = 0; 
	}
	
	// LCD heavy protection mode (D[4]/0x50a3)
	LCD_VREG_LHVLD = protectSelect;
}

/*************************************************************   
 * stopLcdClk function.
 * This function stops LCD clock.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void stopLcdClk(void) {
	// LCD clock enable = disable (D[0]/0x5063)
	OSC_LCLK_LCKEN = 0;
}

/*************************************************************    
 * startLcdClk function.
 * This function sets LCD clock, then starts clock.
 * param	: unsigned char selectOsc		clock source. 0:OSC3  1:OSC1
 * param	: unsigned char clkDivision		clock division ratio when source is OSC3. 0 - 4
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void startLcdClk(unsigned char selectOsc, unsigned char clkDivision) {
	// stop LCD clock
	stopLcdClk();
	
	// LCD clock source select (D[1]/0x5063)
	OSC_LCLK_LCKSRC = selectOsc;
	
	// LCD clock division ratio select (D[4-2]/0x5063)
	OSC_LCLK_LCKDV = clkDivision;

	// LCD clock enable = ensable (D[0]/0x5063)
	OSC_LCLK_LCKEN = 1;

}

/*************************************************************   
 * setLcdContrast function.
 * This function sets LCD contrast.
 * param	: unsigned char contrast		LCD contrast. 0x0 - 0xf
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void setLcdContrast(unsigned char contrast) {
	// LCD contrast adjustment (D[3-0]/0x50a1)
	LCD_CADJ_LC = contrast;
}

/*************************************************************   
 * lcdOn function.
 * This function sets LCD on.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void lcdOn(void) {
	// LCD display control = normal display (D[1-0]/0x50a0)
	LCD_DCTL_DSPC = 0x1;
}

/*************************************************************   
 * lcdOff function.
 * This function sets LCD off.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void lcdOff(void) {
	// LCD display control = display off (D[1-0]/0x50a0)
	LCD_DCTL_DSPC = 0x0;
}

/*************************************************************    
 * lcdAllOn function.
 * This function sets LCD all on.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void lcdAllOn(void) {
	// LCD display control = all on (D[1-0]/0x50a0)
	LCD_DCTL_DSPC = 0x2;
}

/*************************************************************  
 * lcdAllOff function.
 * This function sets LCD all off.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void lcdAllOff(void) {
	// LCD display control = all off (D[1-0]/0x50a0)
	LCD_DCTL_DSPC = 0x3;
}

/*************************************************************   
 * initLcd function.
 * This function initializes LCD.
 * param	: unsigned char selectDuty	display duty. 1:1/16  2:1/32.
 * param	: unsigned char contrast		LCD contrast. 0x0 - 0xf
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void initLcd(unsigned char selectDuty, unsigned char contrast) {
	// LCD off
	lcdOff();
	
	// LCD duty select (D[1-0]/0x50a2)
	LCD_CCTL_LDUTY = selectDuty;
	
	// Display memory area control = area0 (D[5]/0x50a0)
	LCD_DCTL_DSPAR = LCD_DUTY_16_AREA0;
	
	// Segment output assignmend control = normal (D[7]/0x50a0)
	LCD_DCTL_SEGREV = LCD_TERMINAL_ORDER_ASC;

	// Common output assignmend control = normal (D[6]/0x50a0)
	LCD_DCTL_COMREV = LCD_TERMINAL_ORDER_ASC;
	
	// sets LCD contrast
	setLcdContrast(contrast);
	
	// Heverse display control = normal (D[4]/0x50a0)
	LCD_DCTL_DSPREV = LCD_BW_NORMAL;
}

/*************************************************************   
 * reverseLcdArea function.
 * This function reverses LCD display area.
 * When duty is 1/32, this function is not useful.
 * return	: unsigned char  The value of display area which was changed. 0:Area0  1:Area1
 * history	: 2008/04/01	start
 ************************************************************/
unsigned char reverseLcdArea(void) {
	// Display memory area control (D[5]/0x50a0)
	LCD_DCTL_DSPAR = ~LCD_DCTL_DSPAR;
	if(LCD_DCTL_DSPAR) {
		return LCD_DUTY_16_AREA1;
	} else {
		return LCD_DUTY_16_AREA0;
	}
}

/*************************************************************    
 * reverseLcdSeg function.
 * This function reverses LCD segment output order.
 * return	: unsigned char  The value of order which was changed. 0:Descending  1:Ascending
 * history	: 2008/04/01	start
 ************************************************************/
unsigned char reverseLcdSeg(void) {
	// Segment output assignmend control (D[7]/0x50a0)
	LCD_DCTL_SEGREV = ~LCD_DCTL_SEGREV;
	if(LCD_DCTL_SEGREV) {
		return LCD_TERMINAL_ORDER_ASC;
	} else {
		return LCD_TERMINAL_ORDER_DEC;
	}
}

/*************************************************************   
 * reverseLcdCom function.
 * This function reverses LCD common output order.
 * return	: unsigned char  The value of order which was changed. 0:Descending  1:Ascending
 * history	: 2008/04/01	start
 ************************************************************/
unsigned char reverseLcdCom(void) {
	// Common output assignmend control (D[6]/0x50a0)
	LCD_DCTL_COMREV = ~LCD_DCTL_COMREV;
	if(LCD_DCTL_COMREV) {
		return LCD_TERMINAL_ORDER_ASC;
	} else {
		return LCD_TERMINAL_ORDER_DEC;
	}
}

/*************************************************************   
 * reverseLcdBW function.
 * This function reverses LCD monochrome.
 * return	: unsigned char  The value of order which was changed. 0:Reverse  1:Normal
 * history	: 2008/04/01	start
 ************************************************************/
unsigned char reverseLcdBW(void) {
	// Heverse display control (D[4]/0x50a0)
	LCD_DCTL_DSPREV = ~LCD_DCTL_DSPREV;
	if(LCD_DCTL_DSPREV) {
		return LCD_BW_NORMAL;
	} else {
		return LCD_BW_REVERSE;
	}
}

/*************************************************************   
 * initLcdInt function.
 * This function initializes LCD interrupt.
 * param	: unsigned char intLevel		LCD interrupt level. 0 - 7
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void initLcdInt(unsigned char intLevel) {
	// prohibits interrupt
	stopLcdInt();
	
	// LCD interrupt level (D[2-0]/0x430c)
	ITC_LV3_ILV6 = intLevel;
}

/*************************************************************  
 * resetLcdIF function.
 * This function resets interrupt flag.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void resetLcdIF(void) {
	// Frame signal interrupt flag = clear (D[0]/0x50a6)
	LCD_IFLG = 0x01;
}

/*************************************************************   
 * startLcdInt function.
 * This function permits LCD interrupt by IMSK register.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void startLcdInt(void) {
	// resets IF
	resetLcdIF();
	
	// Frame signal interrupt enable = enable (D[0]/0x50a5)
	LCD_IMSK_FRMIE = 1;
}

/*************************************************************  
 * stopLcdInt function.
 * This function prohibits LCD interrupt by IMSK register.
 * return	: void
 * history	: 2008/04/01	start
 ************************************************************/
void stopLcdInt(void) {
	// Frame signal interrupt enable = disable (D[0]/0x50a5)
	LCD_IMSK_FRMIE = 0;
}

